@@ -8,8 +8,8 @@ pymatgen objects for analysis purposes.
 """
 
 __author__ = "Shyue Ping Ong, Dan Gunter"
-__date__ = "Mar 7 2014"
-__version__ = "0.4.2"
+__date__ = "Oct 9 2014"
+__version__ = "0.4.3"
 
 
-from query_engine import QueryEngine
+from .query_engine import QueryEngine
