@@ -5,9 +5,12 @@ __author__ = 'Dan Gunter <dkgunter@lbl.gov>'
 __date__ = '11/4/13'
 
 ## Imports
+import logging
 import matgendb
 # Stdlib
 import os
+# Local
+from matgendb.util import csv_dict, csv_list
 
 ## Globals
 
@@ -59,3 +62,16 @@ def get_schema_file(db_version=1, db="mg_core", collection="materials"):
     schemafile = "{}.{}.json".format(db, collection)
     f = open(os.path.join(d, schemafile), "r")
     return f
+
+def get_builder_log(name):
+    """Get a logging object, in the right place in the
+    hierarchy, for a given builder.
+
+    :param name: Builder name, e.g. 'my_builder'
+    :type name: str
+    :returns: New logger
+    :rtype: logging.Logger
+    """
+    return logging.getLogger("mg.builders." + name)
+
+
