@@ -23,6 +23,8 @@ import string
 import json
 import socket
 import numpy as np
+import zlib
+import six
 from fnmatch import fnmatch
 from collections import OrderedDict
 
@@ -36,11 +38,12 @@ from pymatgen.core.composition import Composition
 from pymatgen.io.vaspio import Vasprun, Incar, Kpoints, Potcar, Poscar, \
     Outcar, Oszicar
 from pymatgen.io.cifio import CifWriter
-from pymatgen.symmetry.finder import SymmetryFinder
+from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
 from pymatgen.analysis.bond_valence import BVAnalyzer
 from monty.io import zopen
 from pymatgen.matproj.rest import MPRester
 from pymatgen.entries.computed_entries import ComputedEntry
+from monty.json import MontyEncoder
 
 
 logger = logging.getLogger(__name__)
@@ -70,9 +73,9 @@ class VaspToDbTaskDrone(AbstractDrone):
 
     def __init__(self, host="127.0.0.1", port=27017, database="vasp",
                  user=None, password=None, collection="tasks",
-                 parse_dos=False, simulate_mode=False,
+                 parse_dos=False, compress_dos=False, simulate_mode=False,
                  additional_fields=None, update_duplicates=True,
-                 mapi_key=None, use_full_uri=True):
+                 mapi_key=None, use_full_uri=True, runs=None):
         """Constructor.
 
         Args:
@@ -92,9 +95,13 @@ class VaspToDbTaskDrone(AbstractDrone):
             collection:
                 Collection to query. Defaults to "tasks".
             parse_dos:
-                Whether to parse the DOS data where possible. Defaults to
-                False. If True, the dos will be inserted into a gridfs
-                collection called dos_fs.
+                Whether to parse the DOS data. Options are True, False, and 'final'
+                Defaults to False. If True, all dos will be inserted into a gridfs
+                collection called dos_fs. If 'final', only the last calculation will
+                be parsed.
+            compress_dos:
+                Whether to compress the DOS data. Valid options are integers 1-9,
+                corresponding to zlib compression level. 1 is usually adequate.
             simulate_mode:
                 Allows one to simulate db insertion without actually performing
                 the insertion.
@@ -122,6 +129,10 @@ class VaspToDbTaskDrone(AbstractDrone):
                 Whether to use full uri path (including hostname) for the
                 directory name. Defaults to True. If False, only the abs
                 path will be used.
+            runs:
+                Ordered list of runs to look for e.g. ["relax1", "relax2"].
+                Automatically detects whether the runs are stored in the
+                subfolder or file extension schema.
         """
         self.host = host
         self.database = database
@@ -130,12 +141,15 @@ class VaspToDbTaskDrone(AbstractDrone):
         self.collection = collection
         self.port = port
         self.simulate = simulate_mode
+        if isinstance(parse_dos, six.string_types) and parse_dos != 'final':
+            raise ValueError('Invalid value for parse_dos')
         self.parse_dos = parse_dos
-        self.additional_fields = {} if not additional_fields \
-            else additional_fields
+        self.compress_dos = compress_dos
+        self.additional_fields = additional_fields or {}
         self.update_duplicates = update_duplicates
         self.mapi_key = mapi_key
         self.use_full_uri = use_full_uri
+        self.runs = runs or ["relax1", "relax2"]
         if not simulate_mode:
             conn = MongoClient(self.host, self.port, j=True)
             db = conn[self.database]
@@ -154,16 +168,14 @@ class VaspToDbTaskDrone(AbstractDrone):
             purposes. Else, only the task_id of the inserted doc is returned.
         """
         try:
-            d = self.get_task_doc(path, self.parse_dos,
-                                  self.additional_fields, self.use_full_uri)
+            d = self.get_task_doc(path)
             if self.mapi_key is not None and d["state"] == "successful":
                 self.calculate_stability(d)
             tid = self._insert_doc(d)
             return tid
         except Exception as ex:
             import traceback
-            print traceback.format_exc(ex)
-            logger.error(traceback.format_exc(ex))
+            logger.error(traceback.format_exc())
             return False
 
     def calculate_stability(self, d):
@@ -179,66 +191,44 @@ class VaspToDbTaskDrone(AbstractDrone):
         for k in ("e_above_hull", "decomposes_to"):
             d["analysis"][k] = data[k]
 
-    @classmethod
-    def get_task_doc(cls, path, parse_dos=False, additional_fields=None,
-                     use_full_uri=True):
+    def get_task_doc(self, path):
         """
         Get the entire task doc for a path, including any post-processing.
         """
         logger.info("Getting task doc for base dir :{}".format(path))
-
-        d = None
-        vasprun_files = OrderedDict()
         files = os.listdir(path)
-        if ("relax1" in files and "relax2" in files and
-                os.path.isdir(os.path.join(path, "relax1")) and
-                os.path.isdir(os.path.join(path, "relax2"))):
-            #Materials project style aflow runs.
-            for subtask in ["relax1", "relax2"]:
-                for f in os.listdir(os.path.join(path, subtask)):
-                    if fnmatch(f, "vasprun.xml*"):
-                        vasprun_files[subtask] = os.path.join(subtask, f)
-        elif "STOPCAR" in files:
+        vasprun_files = OrderedDict()
+        if "STOPCAR" in files:
             #Stopped runs. Try to parse as much as possible.
             logger.info(path + " contains stopped run")
-            for subtask in ["relax1", "relax2"]:
-                if subtask in files and \
-                        os.path.isdir(os.path.join(path, subtask)):
-                    for f in os.listdir(os.path.join(path, subtask)):
-                        if fnmatch(f, "vasprun.xml*"):
-                            vasprun_files[subtask] = os.path.join(
-                                subtask, f)
-        else:
-            vasprun_pattern = re.compile("^vasprun.xml([\w\.]*)")
-            for f in files:
-                m = vasprun_pattern.match(f)
-                if m:
-                    fileext = m.group(1)
-                    if fileext.startswith(".relax2"):
-                        fileext = "relax2"
-                    elif fileext.startswith(".relax1"):
-                        fileext = "relax1"
-                    else:
-                        fileext = "standard"
-                    vasprun_files[fileext] = f
-
-        #Need to sort so that relax1 comes before relax2.
-        sorted_vasprun_files = OrderedDict()
-        for k in sorted(vasprun_files.keys()):
-            sorted_vasprun_files[k] = vasprun_files[k]
+        for r in self.runs:
+            if r in files: #try subfolder schema
+                for f in os.listdir(os.path.join(path, r)):
+                    if fnmatch(f, "vasprun.xml*"):
+                        vasprun_files[r] = os.path.join(r, f)
+            else: #try extension schema
+                for f in files:
+                    if fnmatch(f, "vasprun.xml.{}*".format(r)):
+                        vasprun_files[r] = f
+        if len(vasprun_files) == 0:
+            for f in files: #get any vasprun from the folder
+                if fnmatch(f, "vasprun.xml*") and \
+                        f not in vasprun_files.values():
+                    vasprun_files['standard'] = f
 
         if len(vasprun_files) > 0:
-            d = cls.generate_doc(path, sorted_vasprun_files, parse_dos,
-                                 additional_fields)
+            d = self.generate_doc(path, vasprun_files)
             if not d:
-                d = cls.process_killed_run(path)
-            cls.post_process(path, d, use_full_uri)
+                d = self.process_killed_run(path)
+            self.post_process(path, d)
         elif (not (path.endswith("relax1") or
               path.endswith("relax2"))) and contains_vasp_input(path):
             #If not Materials Project style, process as a killed run.
             logger.warning(path + " contains killed run")
-            d = cls.process_killed_run(path)
-            cls.post_process(path, d, use_full_uri)
+            d = self.process_killed_run(path)
+            self.post_process(path, d)
+        else:
+            raise ValueError("No VASP files found!")
 
         return d
 
@@ -262,7 +252,10 @@ class VaspToDbTaskDrone(AbstractDrone):
                 if self.parse_dos and "calculations" in d:
                     for calc in d["calculations"]:
                         if "dos" in calc:
-                            dos = json.dumps(calc["dos"])
+                            dos = json.dumps(calc["dos"], cls=MontyEncoder)
+                            if self.compress_dos:
+                                dos = zlib.compress(dos, self.compress_dos)
+                                calc["dos_compression"] = "zlib"
                             fs = gridfs.GridFS(db, "dos_fs")
                             dosid = fs.put(dos)
                             calc["dos_fs_id"] = dosid
@@ -274,7 +267,7 @@ class VaspToDbTaskDrone(AbstractDrone):
                         d["task_id"] = db.counter.find_and_modify(
                             query={"_id": "taskid"},
                             update={"$inc": {"c": 1}}
-                        )["c"]
+                            )["c"]
                     logger.info("Inserting {} with taskid = {}"
                                 .format(d["dir_name"], d["task_id"]))
                 elif self.update_duplicates:
@@ -293,8 +286,7 @@ class VaspToDbTaskDrone(AbstractDrone):
                         .format(d["dir_name"], d["task_id"]))
             return d
 
-    @classmethod
-    def post_process(cls, dir_name, d, use_full_uri=True):
+    def post_process(self, dir_name, d):
         """
         Simple post-processing for various files other than the vasprun.xml.
         Called by generate_task_doc. Modify this if your runs have other
@@ -318,14 +310,16 @@ class VaspToDbTaskDrone(AbstractDrone):
         transformations = {}
         filenames = glob.glob(os.path.join(fullpath, "transformations.json*"))
         if len(filenames) >= 1:
-            with zopen(filenames[0], "rb") as f:
+            with zopen(filenames[0], "rt") as f:
                 transformations = json.load(f)
                 try:
                     m = re.match("(\d+)-ICSD",
                                  transformations["history"][0]["source"])
                     if m:
                         d["icsd_id"] = int(m.group(1))
-                except ValueError:
+                except Exception as ex:
+                    logger.warning("Cannot parse ICSD from transformations "
+                                   "file.")
                     pass
         else:
             logger.warning("Transformations file does not exist.")
@@ -352,7 +346,7 @@ class VaspToDbTaskDrone(AbstractDrone):
         # as d["custodian"]
         filenames = glob.glob(os.path.join(fullpath, "custodian.json*"))
         if len(filenames) >= 1:
-            with zopen(filenames[0], "rb") as f:
+            with zopen(filenames[0], "r") as f:
                 d["custodian"] = json.load(f)
 
         # Parse OUTCAR for additional information and run stats that are
@@ -364,7 +358,7 @@ class VaspToDbTaskDrone(AbstractDrone):
                 i = 1 if re.search("relax2", filename) else 0
                 taskname = "relax2" if re.search("relax2", filename) else \
                     "relax1"
-                d["calculations"][i]["output"]["outcar"] = outcar.to_dict
+                d["calculations"][i]["output"]["outcar"] = outcar.as_dict()
                 run_stats[taskname] = outcar.run_stats
         except:
             logger.error("Bad OUTCAR for {}.".format(fullpath))
@@ -382,7 +376,7 @@ class VaspToDbTaskDrone(AbstractDrone):
         d["run_stats"] = run_stats
 
         #Convert to full uri path.
-        if use_full_uri:
+        if self.use_full_uri:
             d["dir_name"] = get_uri(dir_name)
 
         if new_tags:
@@ -390,8 +384,7 @@ class VaspToDbTaskDrone(AbstractDrone):
 
         logger.info("Post-processed " + fullpath)
 
-    @classmethod
-    def process_killed_run(cls, dir_name):
+    def process_killed_run(self, dir_name):
         """
         Process a killed vasp run.
         """
@@ -404,7 +397,7 @@ class VaspToDbTaskDrone(AbstractDrone):
             if fnmatch(f, "INCAR*"):
                 try:
                     incar = Incar.from_file(filename)
-                    d["incar"] = incar.to_dict
+                    d["incar"] = incar.as_dict()
                     d["is_hubbard"] = incar.get("LDAU", False)
                     if d["is_hubbard"]:
                         us = np.array(incar.get("LDAUU", []))
@@ -421,13 +414,13 @@ class VaspToDbTaskDrone(AbstractDrone):
                     else:
                         d["run_type"] = "GGA"
                 except Exception as ex:
-                    print str(ex)
+                    print(str(ex))
                     logger.error("Unable to parse INCAR for killed run {}."
                                  .format(dir_name))
             elif fnmatch(f, "KPOINTS*"):
                 try:
                     kpoints = Kpoints.from_file(filename)
-                    d["kpoints"] = kpoints.to_dict
+                    d["kpoints"] = kpoints.as_dict()
                 except:
                     logger.error("Unable to parse KPOINTS for killed run {}."
                                  .format(dir_name))
@@ -436,7 +429,7 @@ class VaspToDbTaskDrone(AbstractDrone):
                     s = Poscar.from_file(filename).structure
                     comp = s.composition
                     el_amt = s.composition.get_el_amt_dict()
-                    d.update({"unit_cell_formula": comp.to_dict,
+                    d.update({"unit_cell_formula": comp.as_dict(),
                               "reduced_cell_formula": comp.to_reduced_dict,
                               "elements": list(el_amt.keys()),
                               "nelements": len(el_amt),
@@ -444,7 +437,7 @@ class VaspToDbTaskDrone(AbstractDrone):
                               "anonymous_formula": comp.anonymized_formula,
                               "nsites": comp.num_atoms,
                               "chemsys": "-".join(sorted(el_amt.keys()))})
-                    d["poscar"] = s.to_dict
+                    d["poscar"] = s.as_dict()
                 except:
                     logger.error("Unable to parse POSCAR for killed run {}."
                                  .format(dir_name))
@@ -461,7 +454,7 @@ class VaspToDbTaskDrone(AbstractDrone):
             elif fnmatch(f, "OSZICAR"):
                 try:
                     d["oszicar"]["root"] = \
-                        Oszicar(os.path.join(dir_name, f)).to_dict
+                        Oszicar(os.path.join(dir_name, f)).as_dict()
                 except:
                     logger.error("Unable to parse OSZICAR for killed run in {}."
                                  .format(dir_name))
@@ -469,41 +462,39 @@ class VaspToDbTaskDrone(AbstractDrone):
                 if os.path.exists(os.path.join(dir_name, f, "OSZICAR")):
                     try:
                         d["oszicar"][f] = Oszicar(
-                            os.path.join(dir_name, f, "OSZICAR")).to_dict
+                            os.path.join(dir_name, f, "OSZICAR")).as_dict()
                     except:
                         logger.error("Unable to parse OSZICAR for killed "
                                      "run in {}.".format(dir_name))
         return d
 
-    @classmethod
-    def process_vasprun(cls, dir_name, taskname, filename, parse_dos):
+    def process_vasprun(self, dir_name, taskname, filename):
         """
         Process a vasprun.xml file.
         """
         vasprun_file = os.path.join(dir_name, filename)
         r = Vasprun(vasprun_file)
-        d = r.to_dict
+        d = r.as_dict()
         d["dir_name"] = os.path.abspath(dir_name)
         d["completed_at"] = \
             str(datetime.datetime.fromtimestamp(os.path.getmtime(
                 vasprun_file)))
         d["cif"] = str(CifWriter(r.final_structure))
         d["density"] = r.final_structure.density
-        if parse_dos:
+        if self.parse_dos and (self.parse_dos != 'final' \
+                               or taskname == self.runs[-1]):
             try:
-                d["dos"] = r.complete_dos.to_dict
+                d["dos"] = r.complete_dos.as_dict()
             except Exception:
                 logger.warn("No valid dos data exist in {}.\n Skipping dos"
                             .format(dir_name))
         if taskname == "relax1" or taskname == "relax2":
             d["task"] = {"type": "aflow", "name": taskname}
         else:
-            d["task"] = {"type": "standard", "name": "standard"}
+            d["task"] = {"type": taskname, "name": taskname}
         return d
 
-    @classmethod
-    def generate_doc(cls, dir_name, vasprun_files, parse_dos,
-                     additional_fields):
+    def generate_doc(self, dir_name, vasprun_files):
         """
         Process aflow style runs, where each run is actually a combination of
         two vasp runs.
@@ -513,12 +504,11 @@ class VaspToDbTaskDrone(AbstractDrone):
             #Defensively copy the additional fields first.  This is a MUST.
             #Otherwise, parallel updates will see the same object and inserts
             #will be overridden!!
-            d = {k: v for k, v in additional_fields.items()} \
-                if additional_fields else {}
+            d = {k: v for k, v in self.additional_fields.items()}
             d["dir_name"] = fullpath
             d["schema_version"] = VaspToDbTaskDrone.__version__
             d["calculations"] = [
-                cls.process_vasprun(dir_name, taskname, filename, parse_dos)
+                self.process_vasprun(dir_name, taskname, filename)
                 for taskname, filename in vasprun_files.items()]
             d1 = d["calculations"][0]
             d2 = d["calculations"][-1]
@@ -530,10 +520,17 @@ class VaspToDbTaskDrone(AbstractDrone):
                              "is_hubbard", "hubbards", "run_type"]:
                 d[root_key] = d2[root_key]
             d["chemsys"] = "-".join(sorted(d2["elements"]))
-            d["input"] = {"crystal": d1["input"]["crystal"]}
+
+            #store any overrides to the exchange correlation functional
+            xc = d2["input"]["incar"].get("GGA")
+            if xc:
+                xc = xc.upper()
+            d["input"] = {"crystal": d1["input"]["crystal"],
+                          "is_lasph": d2["input"]["incar"].get("LASPH", False),
+                          "xc_override": xc}
             vals = sorted(d2["reduced_cell_formula"].values())
             d["anonymous_formula"] = {string.ascii_uppercase[i]: float(vals[i])
-                                      for i in xrange(len(vals))}
+                                      for i in range(len(vals))}
             d["output"] = {
                 "crystal": d2["output"]["crystal"],
                 "final_energy": d2["output"]["final_energy"],
@@ -545,30 +542,30 @@ class VaspToDbTaskDrone(AbstractDrone):
             d["pseudo_potential"] = {"functional": functional.lower(),
                                      "pot_type": pot_type.lower(),
                                      "labels": d2["input"]["potcar"]}
-            if len(d["calculations"]) == 2 or \
-                    vasprun_files.keys()[0] != "relax1":
+            if len(d["calculations"]) == len(self.runs) or \
+                    list(vasprun_files.keys())[0] != "relax1":
                 d["state"] = "successful" if d2["has_vasp_completed"] \
                     else "unsuccessful"
             else:
                 d["state"] = "stopped"
-
             d["analysis"] = get_basic_analysis_and_error_checks(d)
 
-            sg = SymmetryFinder(Structure.from_dict(d["output"]["crystal"]),
-                                0.1)
-            d["spacegroup"] = {"symbol": unicode(sg.get_spacegroup_symbol(),
-                                                 errors="ignore"),
+            sg = SpacegroupAnalyzer(Structure.from_dict(d["output"]["crystal"]),
+                                    0.1)
+            d["spacegroup"] = {"symbol": sg.get_spacegroup_symbol(),
                                "number": sg.get_spacegroup_number(),
-                               "point_group": unicode(sg.get_point_group(),
-                                                      errors="ignore"),
+                               "point_group": sg.get_point_group(),
                                "source": "spglib",
                                "crystal_system": sg.get_crystal_system(),
                                "hall": sg.get_hall()}
             d["last_updated"] = datetime.datetime.today()
             return d
         except Exception as ex:
+            import traceback
+            print(traceback.format_exc())
             logger.error("Error in " + os.path.abspath(dir_name) +
-                         ".\nError msg: " + str(ex))
+                         ".\n" + traceback.format_exc())
+
             return None
 
     def get_valid_paths(self, path):
@@ -583,11 +580,10 @@ class VaspToDbTaskDrone(AbstractDrone):
            also considered as 2 parts of an aflow style run.
         """
         (parent, subdirs, files) = path
-        if "relax1" in subdirs:
+        if set(self.runs).intersection(subdirs):
             return [parent]
-        if ((not parent.endswith(os.sep + "relax1")) and
-                (not parent.endswith(os.sep + "relax2")) and
-                len(glob.glob(os.path.join(parent, "vasprun.xml*"))) > 0):
+        if not any([parent.endswith(os.sep + r) for r in self.runs]) and \
+                len(glob.glob(os.path.join(parent, "vasprun.xml*"))) > 0:
             return [parent]
         return []
 
@@ -601,8 +597,7 @@ class VaspToDbTaskDrone(AbstractDrone):
     def from_dict(cls, d):
         return cls(**d["init_args"])
 
-    @property
-    def to_dict(self):
+    def as_dict(self):
         init_args = {"host": self.host, "port": self.port,
                      "database": self.database, "user": self.user,
                      "password": self.password,
@@ -673,7 +668,7 @@ def get_basic_analysis_and_error_checks(d, max_force_threshold=0.5,
             "coordination_numbers": coord_num,
             "bandgap": gap, "cbm": cbm, "vbm": vbm,
             "is_gap_direct": is_direct,
-            "bv_structure": bv_struct.to_dict}
+            "bv_structure": bv_struct.as_dict()}
 
 
 def contains_vasp_input(dir_name):
@@ -715,7 +710,7 @@ def get_coordination_numbers(d):
         try:
             n = f.get_coordination_number(i)
             number = int(round(n))
-            cn.append({"site": s.to_dict, "coordination": number})
+            cn.append({"site": s.as_dict(), "coordination": number})
         except Exception:
             logger.error("Unable to parse coordination errors")
     return cn
