@@ -18,6 +18,7 @@ import itertools
 import logging
 import os
 import gridfs
+import zlib
 from collections import OrderedDict, Iterable
 
 from pymongo import MongoClient
@@ -261,7 +262,8 @@ class QueryEngine(object):
         fields = [k for k in optional_data]
         fields.extend(["task_id", "unit_cell_formula", "energy", "is_hubbard",
                        "hubbards", "pseudo_potential.labels",
-                       "pseudo_potential.functional", "run_type"])
+                       "pseudo_potential.functional", "run_type",
+                       "input.is_lasph", "input.xc_override"])
         for c in self.query(fields, criteria):
             func = c["pseudo_potential.functional"]
             labels = c["pseudo_potential.labels"]
@@ -269,7 +271,9 @@ class QueryEngine(object):
             parameters = {"run_type": c["run_type"],
                           "is_hubbard": c["is_hubbard"],
                           "hubbards": c["hubbards"],
-                          "potcar_symbols": symbols}
+                          "potcar_symbols": symbols,
+                          "is_lasph": c.get("input.is_lasph") or False,
+                          "xc_override": c.get("input.xc_override")}
             optional_data = {k: c[k] for k in optional_data}
             if inc_structure:
                 struct = self.get_structure_from_id(c["task_id"])
@@ -310,7 +314,7 @@ class QueryEngine(object):
                 parsed_crit["pretty_formula"] = comp.reduced_formula
             elif key == "unit_cell_formula":
                 comp = Composition(crit)
-                crit = comp.to_dict
+                crit = comp.as_dict()
                 for el, amt in crit.items():
                     parsed_crit["{}.{}".format(self.aliases[key], el)] = amt
                 parsed_crit["nelements"] = len(crit)
@@ -414,9 +418,9 @@ class QueryEngine(object):
         results = tuple(self.query([field], args))
 
         if len(results) > 1:
-            raise QueryError("More than one result found for task_id!")
+            raise QueryError("More than one result found for task_id {}!".format(task_id))
         elif len(results) == 0:
-            raise QueryError("No structure found for task_id!")
+            raise QueryError("No structure found for task_id {}!".format(task_id))
         c = results[0]
         return Structure.from_dict(c[field])
 
@@ -470,13 +474,18 @@ class QueryEngine(object):
         if dosid != None:
             self._fs = gridfs.GridFS(self.db, 'dos_fs')
             with self._fs.get(dosid) as dosfile:
-                d = json.loads(dosfile.read())
+                s = dosfile.read()
+                try:
+                    d = json.loads(s)
+                except:
+                    s = zlib.decompress(s)
+                    d = json.loads(s)
                 tdos = Dos.from_dict(d)
                 pdoss = {}
-                for i in xrange(len(d['pdos'])):
+                for i in range(len(d['pdos'])):
                     ados = d['pdos'][i]
                     all_ados = {}
-                    for j in xrange(len(ados)):
+                    for j in range(len(ados)):
                         orb = Orbital.from_vasp_index(j)
                         odos = ados[str(orb)]
                         all_ados[orb] = {Spin.from_int(int(k)): v
