@@ -11,6 +11,7 @@ import time
 # Package
 from matgendb import util
 from matgendb.query_engine import QueryEngine
+import six
 
 _log = logging.getLogger("mg.vv.diff")
 
@@ -63,7 +64,8 @@ class Differ(object):
         self._info = [] if info is None else info
         self._filter = fltr if fltr else {}
         self._prop_deltas = {} if deltas is None else deltas
-        self._all_props = list(set(self._props[:] + self._prop_deltas.keys()))
+        self._all_props = list(set(self._props[:] +
+                                   list(self._prop_deltas.keys())))
 
     def diff(self, c1, c2, only_missing=False, only_values=False, allow_dup=False):
         """Perform a difference between the 2 collections.
@@ -137,7 +139,7 @@ class Differ(object):
                 # Extract numeric properties.
                 if has_numprops:
                     pvals = {}
-                    for pkey in self._prop_deltas.iterkeys():
+                    for pkey in six.iterkeys(self._prop_deltas):
                         try:
                             pvals[pkey] = float(rec[pkey])
                         except KeyError:
@@ -331,7 +333,7 @@ class Delta(object):
             return {'delta': {'id': self._json_id}}
         dtype = 'abs' if self._eq else 'pct'
         incl = self._eq
-        self._json_id = IID.next()
+        self._json_id = six.advance_iterator(IID)
         return {
             'delta': {
                 'plus': self._dx,
